/*===========================================================*\
    This class models a thread that awakes periodically to
    do a small amount of work.
\*===========================================================*/
#include "MonitorThread.h"

// Body of thread here
int MonitorThread::Run()
{
    // If using a mediator, the code must try/catch the
    // exception type thrown by the mediator.
    try
    {
        while (true)
        {
            // Sleep for awhile then awaken and do something
            // Whenever blocked, always go through the mediator
            Print(Name() + " sleeping");
            _mediator.Sleep(_period);

            DoSomeWork();
        }
    }
    catch(Mediator_t::Exception_t &)
    {
        // Thread was interrupted by mediator; exit gracefully
        Print(Name() + " interrupted");
    }

    return 0;
}



void MonitorThread::DoSomeWork()
{
    Print(Name() + " working");

    int total = 0;
    for (int y = 0; y < 10000; y++)
    {
        for (int x = 0; x < 10000; x++)
            ++total;

        // When actively working, the only sure way to be
        // interrupted is to poll via mediator.
        _mediator.Poll();
    }
}


