/*===========================================================*\
    This class models a thread that waits for input on a socket
\*===========================================================*/
#include "SocketThread.h"
#include "Miscellaneous.h"

// Body of thread here
int SocketThread::Run()
{
    SOCKET socketHandle = ::socket(AF_INET, SOCK_DGRAM, 0);

    // If using a mediator, the code must try/catch the
    // exception type thrown by the mediator.
    try
    {
        while (true)
        {
            Print(Name() + " waiting for data");

            // Wait for input on socket
            _mediator.Wait( 
                    socketHandle, 
                    SocketSynchronizer_t::TIME_INFINITE 
                );

            // If we get here, there is data on the socket
            ReadFromSocket(socketHandle);
        }
    }
    catch(Mediator_t::Exception_t &)
    {
        // Thread was interrupted by mediator; exit gracefully
        Print(Name() + " interrupted");
        ::closesocket(socketHandle);
    }

    return 0;
}


void SocketThread::ReadFromSocket(SOCKET socketHandle)
{
    char buff[16];

    Print(Name() + " reading data");
    int status = ::recvfrom( 
            socketHandle,
            buff,
            sizeof(buff),
            0,
            NULL,
            NULL
        );
}