/*
 * conf.c -- read user's configuration file
 *
 * $Id: conf.c,v 1.3 2003/02/11 16:12:34 danny Exp $
 *
 * THE BEER-WARE LICENSE
 *
 * <danny@alphazed.com> wrote this software. As long as you retain this notice,
 * you can do whatever you want with this stuff.  If we meet some day, and you
 * think this stuff is worth it, you can buy me a beer in return.
 *
 * Daniel Lawrence
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <toolbox.h>
#include "res.h"
#include "sym.h"
#include "conf.h"

char *
conf_eval(FILE *f, RESULT *r, char *msg)
{
	static char *line = NULL;
	static char *buf;	/* dynamic string, but orphaned */
	int err;
	char *cp;
	char *eq;

	while ((line = tb_strgetline(line, f, &err)) != NULL) {
		/* skip white and comments */
		for (cp = line; *cp && isspace(*cp); ++cp)
			;
		if (cp[0] == '\0' || cp[0] == '#')
			continue;

		if (strncmp(cp, "PATH=", 5) == 0) {
			if (!res_path(r, cp+5))
				return tb_strfree(msg);
			continue;
		}

		if (strncmp(cp, "MANPATH=", 8) == 0) {
			if (!res_manpath(r, cp+8))
				return tb_strfree(msg);
			continue;
		}

		if ((eq = strchr(cp, '=')) == NULL) {
			if ((msg = sym_eval(cp, r, msg)) == NULL || msg[0])
				return msg;
			continue;
		}

		/* Dup cp because we will split it and pass pieces on to
		 * res_env(). res_env() does not assume passed strings are
		 * dynamic, so buf is orphaned. This is a memory leak. But
		 * since we just die after displaying the results, there is
		 * no reason to try to free, this program just exits. If it
		 * becomes necessary later to free this memory, we could
		 * probably loop through all the name-value pairs known by
		 * res_env() and free the names.
		 */
		if ((buf = tb_strcpy(NULL, cp)) == NULL)
			return tb_strfree(msg);
		/* terminate variable's name */
		buf[eq-cp] = '\0';
		if (!res_env(r, buf, cp+(eq-cp)+1))
			return tb_strfree(msg);
	}
	if (err == TB_EMEM)
		return tb_strfree(msg);
	if (err == TB_EERRNO)
		return tb_strcpy(msg, strerror(errno));
	if (err != TB_EOK)
		return tb_strprintf(msg, "unknown error %d returned by tb_strgetline", err);
	return tb_strcpy(msg, "");
}
