/*
 * gettok.c
 *
 * $Id: gettok.c,v 1.3 2003/02/11 16:12:34 danny Exp $
 *
 * THE BEER-WARE LICENSE
 *
 * <danny@alphazed.com> wrote this software. As long as you retain this notice,
 * you can do whatever you want with this stuff.  If we meet some day, and you
 * think this stuff is worth it, you can buy me a beer in return.
 *
 * Daniel Lawrence
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "gettok.h"

/* get next whitespace-delimited token */
char *
gettok_white(char **cpp)
{
	char *start;
	char *end;

	/* skip white */
	for (start = *cpp; *start && isspace(*start); ++start)
		;

	/* check for end of string -- no token after all */
	if (*start == '\0') {
		*cpp = start;
		return NULL;
	}

	/* march to end of token */
	for (end = start; *end && !isspace(*end); ++end)
		;

	/* terminate token, advance ptr */
	if (*end != '\0')
		*end++ = '\0';

	*cpp = end;
	return start;
}

/* get next optionally single- or double-quoted token */
char *
gettok_quoted(char **cpp)
{
	char *start;
	char *src;
	char *dst;
	int quote;

	/* skip white */
	for (start = *cpp; *start && isspace(*start); ++start)
		;

	/* check for end of string -- no token after all */
	if (*start == '\0') {
		*cpp = start;
		return NULL;
	}

	/* if first char not quote, degenerate to space-delimited */
	if (*start != '"' && *start != '\'') {
		*cpp = start;
		return gettok_white(cpp);
	}

	/* resulting string will begin where initial quote was */
	dst = src = start;

	/* remember single- or double-quote, advance ptr */
	quote = *src++;

	/* copy and un-escape until terminating quote or \0 */
	while (*src && *src != quote) {
		if (*src == '\\' && *(src+1) != '\0')
			++src;
		*dst++ = *src++;
	}

	/* skip past terminating quote, if it exists */
	if (*src == quote)
		++src;

	/* terminate token */
	*dst++ = '\0';

	*cpp = src;
	return start;
}
