/*
 * readdb.c
 *
 * $Id: readdb.c,v 1.2 2003/02/11 16:12:34 danny Exp $
 *
 * THE BEER-WARE LICENSE
 *
 * <danny@alphazed.com> wrote this software. As long as you retain this notice,
 * you can do whatever you want with this stuff.  If we meet some day, and you
 * think this stuff is worth it, you can buy me a beer in return.
 *
 * Daniel Lawrence
 */

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <toolbox.h>
#include "var.h"
#include "sym.h"
#include "readdb.h"

static int
isempty(char *s)
{
	return (s == NULL || *s == '\0');
}


/* readdb
 *
 * returns:
 *	NULL		out of memory
 *	""		ok, no errors
 *	anything else	error message
 */
char *
readdb(FILE *f, char *msg)
{
	static char *buf = NULL;	/* logical line accumulator */
	static char *line = NULL;
	static char *msg2 = NULL;
	int err;
	int lineno = 0;

	/* clear accumulator (buf) */
	if ((buf = tb_strcpy(buf, "")) == NULL)
		return tb_strfree(msg);

	for (;;) {
		line = tb_strgetline(line, f, &err);
		++lineno;
		if (line == NULL) {
			if (err == TB_EMEM)
				return tb_strfree(msg);
			if (err == TB_EERRNO)
				return tb_strprintf(
					msg,
					"%d: %s",
					lineno,
					strerror(errno)
				);
			/* must be EOF */
			if (!isempty(buf)) {
				/* parse buf, add entry */
				msg2 = sym_parse(buf, msg2);
				if (msg2 == NULL || msg2[0]) {
					return tb_strprintf(
						msg,
						"%d: %s",
						lineno,
						msg2
					);
				}
				return tb_strcpy(msg, "");
			}
		}

		/* skip empty lines and comments */
		if (line[0] == '\0' || line[0] == '#')
			continue;

		/* check for continuation line */
		if (line[0] == ' ' || line[0] == '\t') {
			if (isempty(buf)) {
				return tb_strprintf(
					msg,
					"%d: invalid continuation",
					lineno
				);
			}
		} else {
			/* this is a new entry */
			if (!isempty(buf)) {
				/* flush out current accumulator */
				/* parse buf, add entry */
				msg2 = sym_parse(buf, msg2);
				if (msg2 == NULL || msg2[0]) {
					return tb_strprintf(
						msg,
						"%d: %s",
						lineno,
						msg2
					);
				}
				/* clear accumulator */
				if ((buf = tb_strcpy(buf, "")) == NULL)
					return tb_strfree(msg);
			}
		}
		/* append to accumulator */
		if ((buf = tb_strcat(buf, line, NULL)) == NULL)
			return tb_strfree(msg);
	}
	return tb_strcpy(msg, "can't happen");
}
