/*
 * gethostname.c -- portable gethostname()
 *
 * $Id: gethostname.c,v 1.2 2003/02/11 16:01:45 danny Exp $
 *
 * THE BEER-WARE LICENSE
 *
 * <danny@alphazed.com> wrote this software. As long as you retain this notice,
 * you can do whatever you want with this stuff.  If we meet some day, and you
 * think this stuff is worth it, you can buy me a beer in return.
 *
 * Daniel Lawrence
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "toolbox.h"

#ifdef TEST
/* use small sizes to exercise grow routines */
#define	START_SIZE	2
#define	ADD_SIZE	2
#else
#define	START_SIZE	64
#define	ADD_SIZE	64
#endif


char *
tb_gethostname(void)
{
	char *buf;
	char *newbuf;
	int size;
	int i;

	buf = NULL;
	for (;;) {
		/* make a bigger buffer */
		if (buf == NULL) {
			if ((buf = malloc(START_SIZE)) == NULL)
				break;
			size = START_SIZE;
		} else {
			if ((newbuf = realloc(buf, size + ADD_SIZE)) == NULL) {
				free(buf);
				buf = NULL;
				break;
			}
			buf = newbuf;
			size += ADD_SIZE;
		}

		/* make every char non-nul so we can detect if gethostname()
		 * terminated the buffer
		 */
		memset(buf, ' ', size);

		/* if gethostname returns any error, we automatically know
		 * the buffer wasn't big enough, so retry with bigger buf
		 */
		if (gethostname(buf, size) == -1)
			continue;

		/* scan buffer for nul if it exists */
		for (i = 0; i < size; ++i)
			if (buf[i] == '\0')
				break;

		/* if there is no nul, then result was truncated or not set
		 * at all. either way indicates the buffer was too small, so
		 * retry with bigger buf
		 */
		if (i == size)
			continue;

		/* if the nul is the last character of buffer, then the result
		 * may have been truncated or the hostname may be exactly
		 * size-1 characters long. only way to tell is to retry with a
		 * bigger buf
		 */
		if (i == size-1)
			continue;

		/* result is nul-terminated and shorter than buffer. it must
		 * have worked
		 */
		break;
	}

	return buf;
}
