/*
 * mkdirp -- make directory and all intermediate directories if necessary
 *
 * $Id: mkdirp.c,v 1.2 2003/02/11 16:01:45 danny Exp $
 *
 * THE BEER-WARE LICENSE
 *
 * <danny@alphazed.com> wrote this software. As long as you retain this notice,
 * you can do whatever you want with this stuff.  If we meet some day, and you
 * think this stuff is worth it, you can buy me a beer in return.
 *
 * Daniel Lawrence
 */

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <unistd.h>
#include "toolbox.h"


static int
md(char *path, uid_t uid, gid_t gid, mode_t mode, int last, char **msgp)
{
	struct stat sb;

	if (*path == '\0' || path[strlen(path)-1] == '/')
		return 1;

	if (stat(path, &sb) == 0) {
		if (sb.st_mode & S_IFDIR)
			return 1;
		*msgp = tb_strprintf(
			*msgp,
			"%s already exists but is not a directory",
			path
		);
		return 0;
	}
	if (errno != ENOENT) {
		*msgp = tb_strprintf(
			*msgp,
			"%s: can't stat:",
			path
		);
		return 0;
	}
	if (mkdir(path, 0777) == -1) {
		*msgp = tb_strprintf(
			*msgp,
			"%s: can't mkdir:",
			path
		);
		return 0;
	}
	if (stat(path, &sb) == -1) {
		*msgp = tb_strprintf(
			*msgp,
			"%s: can't stat %s:",
			path
		);
		return 0;
	}
	/* If we are not on the last directory component, simply ensure
	 * that the modes will allow us to create a subdirectory. (o+wx)
	 * But if we are on the final directory, use the modes provided
	 * by the user.
	 *
	 * chown before chmod because chown may clear suid and sgid bits.
	 */
	if (!last)
		mode = sb.st_mode | S_IWUSR | S_IXUSR;
	else {
		if ((uid != (uid_t)-1 || gid != (gid_t)-1)
		    && (sb.st_uid != uid || sb.st_gid != gid)) {
			if (chown(path, uid, gid) == -1) {
				*msgp = tb_strprintf(
					*msgp,
					"%s: can't chown:",
					path
				);
				return 0;
			}
		}
	}

	if (mode != (mode_t)-1) {
		if (chmod(path, mode) == -1) {
			*msgp = tb_strprintf(
				*msgp,
				"%s: can't chmod %s:",
				path
			);
			return 0;
		}
	}
	return 1;
}


int
tb_mkdirp(char *path, uid_t uid, gid_t gid, mode_t mode, char **msgp)
{
	static char *buf = NULL;
	char *cp;

	/* make working copy just in case we were passed a string constant */
	if ((buf = tb_strcpy(buf, path)) == NULL)
		return 0;

	/* remove any trailing slashes so we know which is the last dir */
	tb_rtrimc(buf, '/');
		
	for (cp = buf; *cp; ++cp)
		if (cp != buf && *cp == '/') {
			*cp = '\0';
			if (!md(buf, uid, gid, mode, 0, msgp)) {
				*cp = '/';
				return 0;
			}
			*cp = '/';
		}

	if (cp != buf)
		return md(buf, uid, gid, mode, 1, msgp);
	return 1;
}
