/*
 * mktmp.c -- create temporary file (not re-entrant)
 *
 * $Id: mktmp.c,v 1.2 2003/02/11 16:01:45 danny Exp $
 *
 * THE BEER-WARE LICENSE
 *
 * <danny@alphazed.com> wrote this software. As long as you retain this notice,
 * you can do whatever you want with this stuff.  If we meet some day, and you
 * think this stuff is worth it, you can buy me a beer in return.
 *
 * Daniel Lawrence
 *
 * Filenaming convention taken from http://cr.yp.to/proto/maildir.html
 */

#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/time.h>
#include <fcntl.h>
#include <errno.h>
#include "toolbox.h"


int
tb_mktmp(char *prefix, char **pathp, int *err)
{
	static unsigned long gen = 0;
	static char *hostname = NULL;
	pid_t pid;
	struct timeval tv;
	int fd;

	/* cache hostname */
	if (hostname == NULL)
		if ((hostname = tb_gethostname()) == NULL) {
			*err = TB_EMEM;
			return -1;
		}

	pid = getpid();

	for (;;) {
		if (gettimeofday(&tv, NULL) == -1) {
			*err = TB_EERRNO;
			return -1;
		}

		*pathp = tb_strprintf(
			*pathp,
			"%s%ld.M%ldP%luQ%ld.%s",
			prefix,
			tv.tv_sec,
			tv.tv_usec,
			(unsigned long)pid,
			gen,
			hostname
		);
		if (*pathp == NULL) {
			*err = TB_EMEM;
			return -1;
		}

		if ((fd = open(*pathp, O_RDWR|O_CREAT|O_EXCL, 0600)) != -1) {
			++gen;
			return fd;
		}

		if (errno == EEXIST) {
			/* collission */
			sleep(2);
			continue;
		}

		*err = TB_EERRNO;
		return -1;
	}
}

