.\" salt.man
.\" $Id: salt.man,v 1.1 2003/02/25 13:32:12 danny Exp $
.TH tb_salt 3
.SH NAME
tb_salt - generate salt for crypt()
.SH LIBRARY
Toolbox (-ltoolbox)
.SH SYNOPSIS
.nf
#include <toolbox.h>

char *
tb_salt(char *salt, char *template, int (pick)(int));

char *
tb_salt_err(void);
.fi
.SH DESCRIPTION
tb_salt() generates a salt string for crypt().
template describes what the salt should look like.
pick() is a random number generator which can optionally be used instead of
the default based on rand().
.PP
The template provided to salt looks like a regular expression.
Literal characters are copied without interpretation, but characters and
ranges between [ and ] cause a random character to be produced.
An optional count between { and } causes that many random characters from the
preceding range to be produced.
Any number of individual characters and ranges may be concatenated between
the [ and ] like this: "[A-Za-z./]".
.PP
Traditional crypt() salts would use the template "[A-Za-z0-9./]{2}".
This is the default, if a NULL template is passed to tb_salt().
.PP
"DES extended" salts would use the template "_[./0-9A-Za-z]{8}".
.PP
Modular MD5 salts would use the template "$1$[A-Za-z0-9./]{8}" or sometimes
"$1$[A-Za-z0-9./]{16}".
.PP
Characters can be escaped by preceding them with a backslash.
For example, if you need to put a ] in the range, you can escape it like
this: "[A-Z\\]]".
Arbitrary characters can be included in the template by escaping the octal
value of the character.
For example, a right bracket could be escaped "\\135".
.PP
The pick function passed to tb_salt() takes a single integer argument n and
returns a random integer in the range 0..n-1.
If NULL is passed, the default function is used:
.sp
rand() % n
.sp
The rand() function is not very strong, so it is advisable to provide your
own function which uses a better random number generator.
.SH RETURN VALUES
If tb_salt() is successful, the salt is returned as a dynamic string.
Otherwise NULL is returned and tb_salt_err() can be called to retrieve the
error message.
.SH EXAMPLES
The basic way to generate a traditional salt:
.sp
.nf
#include <toolbox.h>

static char *salt = NULL;

salt = tb_salt(salt, NULL, NULL);
if (salt == NULL)
	printf("%s\\n", tb_salt_err());
else
	printf("salt is %s\\n", salt);
.fi
.sp
To generate an MD5 salt using a better random number generator:
.sp
.nf
#include <toolbox.h>

static char *salt = NULL;

int
generator(int n)
{
	/* return random number in range 0..n-1 */
}

salt = tb_salt(salt, "$1$[A-Za-z0-9./]{16}", generator);
.fi
.sp
To generate a random alphanumeric password for a new user:
.sp
.nf
password = tb_salt(password, "[A-Za-z0-9]{8}", NULL);
.fi
.SH SEE ALSO
crypt(3),
rand(3),
.br
http://www.alphazed.co.uk/programming/salt/
.SH BUGS
tb_salt() and tb_salt_err() use static variables and are not re-entrant.
