/*
 * string.c -- basic string management routines (private)
 *		(simple-minded implementation)
 *
 * $Id: string.c,v 1.3 2003/02/11 16:01:45 danny Exp $
 *
 * THE BEER-WARE LICENSE
 *
 * <danny@alphazed.com> wrote this software. As long as you retain this notice,
 * you can do whatever you want with this stuff.  If we meet some day, and you
 * think this stuff is worth it, you can buy me a beer in return.
 *
 * Daniel Lawrence
 */

#include <string.h>
#include <stdlib.h>
#include "tb__string.h"


tb__string *tb__strtab = NULL;
int tb__strtabsize = 0;
unsigned tb__strlookups = 0;
unsigned tb__strcompares = 0;


/* allocate a new tb__string, set to optional initial string */
tb__string *
tb__stralloc(char *cp)
{
	tb__string *p;
	int i;

	if (tb__strtabsize == 0) {
		/* first time, allocate new table */
		/* start at one element to make sure grow routine is */
		/* exercised */
		if ((p = malloc(sizeof(tb__string))) == NULL)
			return NULL;
		p->s = NULL;
		p->size = 0;
		tb__strtab = p;
		tb__strtabsize = 1;
		i = 0;
	} else {
		/* search for empty element in table */
		for (i = 0; i < tb__strtabsize; ++i)
			if (tb__strtab[i].s == NULL)
				break;
	}

	if (i == tb__strtabsize) {
		/* no empty elements in table, grow table */
		p = realloc(
			tb__strtab,
			2 * tb__strtabsize * sizeof(tb__string)
		);
		if (p == NULL)
			return NULL;
		tb__strtab = p;
		for (i = tb__strtabsize; i < 2 * tb__strtabsize; ++i) {
			tb__strtab[i].s = NULL;
			tb__strtab[i].size = 0;
		}
		i = tb__strtabsize;
		tb__strtabsize *= 2;
	}

	if ((tb__strtab[i].s = strdup(cp)) == NULL)
		return NULL;
	tb__strtab[i].size = strlen(tb__strtab[i].s) + 1;

	return tb__strtab + i;
}


/* look for a string in table */
tb__string *
tb__strlookup(char *cp)
{
	int i;

	++tb__strlookups;
	for (i = 0; i < tb__strtabsize; ++i) {
		++tb__strcompares;
		if (tb__strtab[i].s == cp)
			return tb__strtab + i;
	}
	return NULL;
}


/* free string and clear its table entry */
void
tb__strfree(tb__string *p)
{
	free(p->s);
	p->s = NULL;
	p->size = 0;
}
