/*
 * strprintf.c
 *
 * $Id: strprintf.c,v 1.2 2003/02/11 16:01:45 danny Exp $
 *
 * THE BEER-WARE LICENSE
 *
 * <danny@alphazed.com> wrote this software. As long as you retain this notice,
 * you can do whatever you want with this stuff.  If we meet some day, and you
 * think this stuff is worth it, you can buy me a beer in return.
 *
 * Daniel Lawrence
 */

#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include "toolbox.h"
#include "tb__string.h"

char *
tb_strprintf(char *buf, char *fmt, ...)
{
	va_list args;
	tb__string *p;
	size_t size;
	size_t newsize;
	char *news;

	if (buf == NULL) {
		if ((p = tb__stralloc("")) == NULL)
			return NULL;
	} else if ((p = tb__strlookup(buf)) == NULL) {
		fprintf(stderr, "tb_strprintf: martian\n");
		abort();
	}

	for (;;) {
		va_start(args, fmt);
		size = vsnprintf(p->s, p->size, fmt, args);
		va_end(args);

		if (size == -1) {
			/* Clear indication that output was truncated, but no
			 * clear indication of how big buffer needs to be, so
			 * simply double existing buffer size for next time.
			 */
			newsize = p->size * 2;

		} else if (size == p->size) {
			/* Output was truncated (since at least the \0 could
			 * not fit), but no indication of how big the buffer
			 * needs to be, so just double existing buffer size
			 * for next time.
			 */
			newsize = p->size * 2;

		} else if (size > p->size) {
			/* Output was truncated, but we were told exactly how
			 * big the buffer needs to be next time. Add two chars
			 * to the returned size. One for the \0, and one to
			 * prevent ambiguity in the next case below.
			 */
			newsize = size + 2;

		} else if (size == p->size - 1) {
			/* This is ambiguous. May mean that the output string
			 * exactly fits, but on some systems the output string
			 * may have been truncated. We can't tell.
			 * Just double the buffer size for next time.
			 */
			newsize = p->size * 2;

		} else {
			/* Output was not truncated */
			break;
		}

		if ((news = realloc(p->s, newsize)) == NULL) {
			tb__strfree(p);
			return NULL;
		}
		p->s = news;
		p->size = newsize; 
	}

	return p->s;
}
