/**************************************************************************
 * Copyright (c) 2002 Edward Elliott. All rights reserved.
 *
 * Threading Policies - classes which provide synchronization when
 * necessary.
 *************************************************************************/

#ifndef THREADPOL_H
#define THREADPOL_H

/**
 * Dummy policy for single-threaded programs
 */
template <class Target>
class NoLocking
{
public:
    struct Lock
    {
        Lock() {}
        Lock(const Target&) {}
    };

	typedef Target	VolatileType;
};


// Define this flag during compile if you don't want to use Boost threads
#ifndef  NO_BOOST_THREADS

#include <boost/thread/thread.hpp>

/**
 * Mutex-locking policy for multi-threaded programs
 */
template <class Target>
class MutexLocking
{
public:
	class Lock;
	friend class Lock;

	class Lock
	{
	public:
		Lock() : lock_(mtx_)	{}
		Lock(Target&) : lock_(mtx_)	{}
	private:
		boost::mutex::scoped_lock	lock_;
	};

	typedef volatile Target VolatileType;

private:
	static boost::mutex	mtx_;
};

template<class Target>
typename boost::mutex MutexLocking<Target>::mtx_;

#endif	// NO_BOOST_THREADS

#endif	// THREADPOL_H
