/**************************************************************************
 * Copyright (c) 2002 Edward Elliott. All rights reserved.
 *
 * Tests the dlmalloc library calls.
 *************************************************************************/

#include "dlmalloc.h"
#include <set>
#include <vector>
#include <iostream>
#include <iomanip>
#include <iterator>
#include <cstdlib>

using namespace std;

int main(int argc, char* argv[])
{
	typedef vector<void*>	MemBlocks;
	MemBlocks mallocs, dlmallocs;

	typedef set<size_t>	Pages;
	Pages	mallocPages;
	Pages	dlmallocPages;

	const size_t pageSize = dlgetpagesize();
	cout << "Page Size: " << pageSize / 1024 << "k" << endl;

	// Allocate some memory blocks with malloc and dlmalloc,
	// successively doubling the block size.  Record the page(s)
	// touched by each allocation.
	const size_t maxSize = 2 * pageSize;
	for (size_t size = 1; size <= maxSize; size *= 2)
	{
		void *mem = malloc(size);
		mallocs.push_back( mem );
		mallocPages.insert( size_t(mem) / pageSize );
		mallocPages.insert( (size_t(mem) + size) / pageSize );

		void *dlmem = dlmalloc(size);
		dlmallocs.push_back( dlmem );
		dlmallocs.push_back( dlmem );
		dlmallocPages.insert( size_t(dlmem) / pageSize );
		dlmallocPages.insert( (size_t(dlmem) + size) / pageSize );
	}

	cout << "Pages used by malloc: " << endl;
	cout.setf(ios::uppercase | ios::showbase);
	cout << "    ";// << uppercase << showbase << hex;
	copy(mallocPages.begin(), mallocPages.end(), 
		ostream_iterator<size_t>(cout, " ") );
	cout << endl;

	cout << "Pages used by dlmalloc: " << endl;
	cout << "    ";
	copy(dlmallocPages.begin(), dlmallocPages.end(),
		ostream_iterator<size_t>(cout, " ") );
	cout << endl;

	// Compare the pages touched by malloc and dlmalloc, making
	// sure there is no overlap.
	Pages::iterator i;
	bool overlap = false;
	for (i = mallocPages.begin(); i != mallocPages.end(); ++i)
	{
		if (dlmallocPages.find(*i) != dlmallocPages.end())
		{
			overlap = true;
			cout << "Warning: dlmalloc and malloc allocated from "
					"the same page: " << *i << endl;
		}
	}

	cout << (overlap ? "Failure" : "Success") << endl;
	return 0;
}
