/**************************************************************************
 * Copyright (c) 2002 Edward Elliott. All rights reserved.
 *
 * Compiling this file should instantiate all templates and template
 * functions in the secure allocator policies (for the current platform
 * at least).  When run, it will call the four SecureAllocator functions
 * of interest and use the allocator in a char vector.
 *************************************************************************/

#include "SecAlloc.h"
#include <iostream>
#include <iomanip>
#include <vector>
#include <set>

using namespace std;

int main(int argc, char* argv[])
{
	cout << "Testing malloc/noscrub/nolock: ";
	cout << flush;

	typedef
		SecureAllocator<char, ScrubOnDeallocate<NoScrubber>, JustPlainMalloc,
		NoLocking<JustPlainMalloc> >
	BasicAlloc;

	{
		BasicAlloc ba;
		BasicAlloc::pointer p;
		ba.max_size();
		p = ba.allocate(1);
		ba.construct(p, 'a');
		ba.destroy(p);
		ba.deallocate(p, 1);

		vector<char, BasicAlloc> w;
		w.push_back('a');

		set<char, less<char>, BasicAlloc> s;
		s.insert('a');
	}

	cout << "success" << endl;
	cout << "Testing noswap/basicscrub/nolock: ";
	cout << flush;

	typedef 
		SecureAllocator<char, ScrubOnDestroy<BasicScrubber>, NoSwapAlloc,
		NoLocking<NoSwapAlloc> >
	SSecAlloc;

	{
		SSecAlloc ssa;
		SSecAlloc::pointer p;
		ssa.max_size();
		p = ssa.allocate(1);
		ssa.construct(p, 'a');
		ssa.destroy(p);
		ssa.deallocate(p, 1);

		vector<char, SSecAlloc> v;
		v.push_back('a');

		set<char, less<char>, SSecAlloc> t;
		t.insert('a');
	}

	cout << "success" << endl;

// Define this to disable the use of Boost mutexes
#ifndef  NO_BOOST_THREADS

	cout << "Testing noswap/basicscrub/mutex: ";
	cout << flush;

	typedef 
		SecureAllocator<char, ScrubOnDestroy<BasicScrubber>, NoSwapAlloc,
		MutexLocking<NoSwapAlloc> >
	MSecAlloc;

	{
		MSecAlloc msa;
		MSecAlloc::pointer p;
		msa.max_size();
		p = msa.allocate(1);
		msa.construct(p, 'a');
		msa.destroy(p);
		msa.deallocate(p, 1);

		vector<char, MSecAlloc> v;
		v.push_back('a');

		set<char, less<char>, MSecAlloc> t;
		t.insert('a');
	}

	cout << "success" << endl;

#endif	// NO_BOOST_THREADS

	return 0;
}
